#include <stdlib.h>
#include <unistd.h>
#include <tracefs.h>

struct cpu_info {
	unsigned long long	start;
	unsigned long long	irq_vec;
	char			*irq_name;
};

struct irq_context {
	struct tep_handle	*tep;
	struct tep_event	*func_enter;
	struct tep_event	*func_exit;
	struct tep_event	*irq;
	struct tep_format_field *irq_vec;
	struct tep_format_field *irq_name;
	struct cpu_info		max;
	int			nr_cpus;
	struct cpu_info		*cpus;
	struct trace_seq	seq;
	int			max_cpu;
};

static int callback(struct tep_event *event, struct tep_record *record,
		    int cpu, void *data);

int main (int argc, char **argv)
{
	struct irq_context context;

	context.tep = tracefs_local_events(NULL);
	tep_set_cpus(context.tep, sysconf(_SC_NPROCESSORS_CONF));
	context.nr_cpus = tep_get_cpus(context.tep);
	context.cpus = calloc(context.nr_cpus, sizeof(struct cpu_info));
	context.func_enter = tep_find_event_by_name(context.tep, "ftrace", "funcgraph_entry");
	context.func_exit = tep_find_event_by_name(context.tep, "ftrace", "funcgraph_exit");
	context.irq = tep_find_event_by_name(context.tep, "irq", "irq_handler_entry");
	context.irq_vec = tep_find_field(context.irq, "irq");
	context.irq_name = tep_find_field(context.irq, "name");
	memset(&context.max, 0, sizeof(context.max));
	trace_seq_init(&context.seq);

	tracefs_trace_off(NULL);
	tracefs_function_filter(NULL, "*common_interrupt*", NULL, TRACEFS_FL_RESET);
	tracefs_tracer_set(NULL, TRACEFS_TRACER_FUNCTION_GRAPH);
	tracefs_event_disable(NULL, NULL, NULL);
	tracefs_event_enable(NULL, "irq", "irq_handler_entry");
	tracefs_trace_on(NULL);
	sleep(5);
	tracefs_trace_off(NULL);

	tracefs_iterate_raw_events(context.tep, NULL, NULL, 0, callback, &context);

	if (context.max.start)
		printf("Max irq latency was %lld us from irq %s:%lld on cpu %d\n",
		       context.max.start / 1000, context.max.irq_name, context.max.irq_vec, context.max_cpu);
	else
		printf("No interrupt was recorderd ;-(\n");
	tracefs_tracer_clear(NULL);
	tracefs_function_filter(NULL, NULL, NULL, TRACEFS_FL_RESET);
	tracefs_event_disable(NULL, NULL, NULL);
	return 0;
}

static int callback(struct tep_event *event, struct tep_record *record,
		    int cpu, void *data)
{
	unsigned long long delta;
	struct irq_context *context = data;
	int type = tep_data_type(context->tep, record);

	if (cpu >= context->nr_cpus)
		return 0;

	if (type == context->irq->id) {
		if (context->cpus[cpu].start) {
			trace_seq_reset(&context->seq);
			tep_print_field_content(&context->seq, record->data,
						record->size, context->irq_name);
			free(context->cpus[cpu].irq_name);
			trace_seq_terminate(&context->seq);
			context->cpus[cpu].irq_name = strdup(context->seq.buffer);
			tep_read_number_field(context->irq_vec, record->data,
					      &context->cpus[cpu].irq_vec);
		}
	} else if (type == context->func_enter->id) {
		context->cpus[cpu].start = record->ts;

	} else if (type == context->func_exit->id) {
		if (context->cpus[cpu].start) {
			delta = record->ts - context->cpus[cpu].start;
			if (delta > context->max.start) {
				context->max.start = delta;
				context->max.irq_vec = context->cpus[cpu].irq_vec;
				context->max.irq_name = context->cpus[cpu].irq_name;
				context->cpus[cpu].irq_name = NULL;
				context->max_cpu = cpu;
			}
			context->cpus[cpu].start = 0;
		}
	}
		
	return 0;
}
