#include <event-parse.h>
#include <trace-seq.h>

struct func_ips {
	struct tep_format_field		*ip;
	struct tep_format_field		*pip;
};

static struct func_ips func_ips;

static int pfunc_index(struct trace_seq *seq, struct tep_record *record,
		struct tep_event *event, void *context)
{
	struct tep_handle *tep = event->tep;
	struct func_ips *fips = context;
	unsigned long long ip, pip;

	tep_read_number_field(fips->ip, record->data, &ip);
	tep_read_number_field(fips->pip, record->data, &pip);

	trace_seq_printf(seq, "%s <-- %s+%lld",
			 tep_find_function(tep, ip),
			 tep_find_function(tep, pip),
			 pip - tep_find_function_address(tep, pip));

	return 0;
}

int TEP_PLUGIN_LOADER(struct tep_handle *tep)
{
	struct tep_event *func_event;
	func_event = tep_find_event_by_name(tep, "ftrace", "function");
	func_ips.ip = tep_find_field(func_event, "ip");
	func_ips.pip = tep_find_field(func_event, "parent_ip");
	tep_register_event_handler(tep, -1, "ftrace", "function", pfunc_index, &func_ips);
	return 0;
}
void TEP_PLUGIN_UNLOADER(struct tep_handle *tep)
{
	tep_unregister_event_handler(tep, -1, "ftrace", "function", pfunc_index, &func_ips);
}
