#define _GNU_SOURCE
#include <stdlib.h>
#include <unistd.h>
#include <sched.h>
#include <errno.h>
#include <trace-cmd.h>

static int print_record(struct tep_handle *tep, struct tep_record *record,
			struct trace_seq *seq)
{
	int cpu = record->cpu;

	trace_seq_reset(seq);
	tep_print_event(tep, seq, record, "%6.1000d", TEP_PRINT_TIME);
	trace_seq_printf(seq, " [%03d] ", cpu);
	tep_print_event(tep, seq, record, "%s-%d %s %s\n",
			TEP_PRINT_COMM, TEP_PRINT_PID,
			TEP_PRINT_NAME, TEP_PRINT_INFO);
	trace_seq_do_printf(seq);
	return 0;
}

static struct tep_record *get_next_record(struct tep_handle *tep,
					  struct tracecmd_input *handle)
{
	static struct tep_record **cpu_records;
	struct tep_record *record;
	unsigned long long ts = -1ULL;
	int nr_cpus;
	int next = -1;
	int i;

	nr_cpus = tep_get_cpus(tep);

	if (!cpu_records) {
		cpu_records = calloc(nr_cpus, sizeof(*cpu_records));
		for (i = 0; i < nr_cpus; i++)
			cpu_records[i] = tracecmd_read_cpu_first(handle, i);
	}

	for (i = 0; i < nr_cpus; i++) {
		if (cpu_records[i] && cpu_records[i]->ts < ts)
			next = i;
	}
	if (next < 0)
		return NULL;

	record = cpu_records[next];
	cpu_records[next] = tracecmd_read_data(handle, next);

	return record;
}

int main(int argc, char **argv) {
	struct tracecmd_input *handle, *pi_handle;
	struct tep_record *record;
	struct tep_handle *tep;
	struct trace_seq seq;
	char *file = "trace.dat";
	int i, nr_instances;

	if (argc > 1)
		file = argv[1];

	tracecmd_set_loglevel(TEP_LOG_CRITICAL);

	handle = tracecmd_open(file, 0);
	nr_instances = tracecmd_buffer_instances(handle);
	for (i = 0; i < nr_instances; i++) {
		if (strcmp("pi-test", tracecmd_buffer_instance_name(handle, i)) == 0)
			break;
	}
	pi_handle = tracecmd_buffer_instance_handle(handle, i);

	tep = tracecmd_get_tep(pi_handle);
	trace_seq_init(&seq);

	while ((record = get_next_record(tep, pi_handle)) != NULL) {
		print_record(tep, record, &seq);
		tracecmd_free_record(record);
	}
	trace_seq_destroy(&seq);

	return 0;
}
