#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <event-parse.h>
#include <kbuffer.h>
#include <trace-seq.h>

static char *read_file(const char *file)
{
	char buf[BUFSIZ];
	char *f = NULL;
	int fd, r, size = 0;

	fd = open(file, O_RDONLY);
	do {
		r = read(fd, buf, BUFSIZ);
		f = realloc(f, r + size); // unsafe!
		memcpy(f + size, buf, r);
		size += r;
	} while (r > 0);

	return f;
}

int main(int argc, char **argv) {
	struct tep_record record;
	struct tep_handle *tep;
	struct tep_event *func_event;
	struct tep_format_field *func_ip;
	struct tep_format_field *func_pip;
	struct kbuffer *kbuf;
	struct trace_seq seq;
	unsigned long long ts;
	void *buf;
	int sub_buf_size;
	int fd;

	fd = open(argv[1], O_RDONLY);

	trace_seq_init(&seq);
	tep = tep_alloc();
	buf = read_file("/sys/kernel/tracing/events/header_page");
	tep_parse_header_page(tep, buf, strlen(buf), 0);
	free(buf);
	buf = read_file("/sys/kernel/tracing/events/sched/sched_switch/format");
	tep_parse_event(tep, buf, strlen(buf), "sched");
	free(buf);
	buf = read_file("/sys/kernel/tracing/events/ftrace/function/format");
	tep_parse_event(tep, buf, strlen(buf), "ftrace");
	free(buf);
	func_event = tep_find_event_by_name(tep, "ftrace", "function");
	func_ip = tep_find_field(func_event, "ip");
	func_pip = tep_find_field(func_event, "parent_ip");
	buf = read_file("/proc/kallsyms");
	tep_parse_kallsyms(tep, buf);
	free(buf);
	buf = read_file("/sys/kernel/tracing/saved_cmdlines");
	tep_parse_saved_cmdlines(tep, buf);
	free(buf);
	sub_buf_size = tep_get_sub_buffer_size(tep);
	buf = malloc(sub_buf_size);
	read(fd, buf, sub_buf_size);
	kbuf = kbuffer_alloc(tep_get_header_page_size(tep) == 8, !tep_is_bigendian());
	kbuffer_load_subbuffer(kbuf, buf);
	for (;;) {
		record.data = kbuffer_read_event(kbuf, &ts);
		if (!record.data)
			break;
		record.ts = ts;
		record.missed_events = kbuffer_missed_events(kbuf);
		record.size = kbuffer_event_size(kbuf);
		record.record_size = kbuffer_curr_size(kbuf);
		record.cpu = 0;
		kbuffer_next_event(kbuf, NULL);
		trace_seq_reset(&seq);
		if (tep_data_type(tep, &record) == func_event->id) {
			unsigned long long ip, pip;
			tep_print_event(tep, &seq, &record, "%6.1000d %s-%d %s ",
					TEP_PRINT_TIME, TEP_PRINT_COMM, TEP_PRINT_PID,
					TEP_PRINT_NAME);
			tep_read_number_field(func_ip, record.data, &ip);
			tep_read_number_field(func_pip, record.data, &pip);

			trace_seq_printf(&seq, "%s <-- %s+%lld\n",
					 tep_find_function(tep, ip),
					 tep_find_function(tep, pip),
					 pip - tep_find_function_address(tep, pip));
		} else
			tep_print_event(tep, &seq, &record, "%6.1000d %s-%d %s %s\n",
					TEP_PRINT_TIME, TEP_PRINT_COMM, TEP_PRINT_PID,
					TEP_PRINT_NAME, TEP_PRINT_INFO);
		trace_seq_do_printf(&seq);
	}
	trace_seq_destroy(&seq);

	return 0;
}


