#define _GNU_SOURCE
#include <stdlib.h>
#include <unistd.h>
#include <tracefs.h>
#include <sched.h>
#include <errno.h>

static int callback(struct tep_event *event, struct tep_record *record,
		    int cpu, void *data)
{
	struct trace_seq *seq = data;

	trace_seq_reset(seq);
	tep_print_event(event->tep, seq, record, "%6.1000d", TEP_PRINT_TIME);
	trace_seq_printf(seq, " [%03d] ", cpu);
	tep_print_event(event->tep, seq, record, "%s-%d %s %s\n",
			TEP_PRINT_COMM, TEP_PRINT_PID,
			TEP_PRINT_NAME, TEP_PRINT_INFO);
	trace_seq_do_printf(seq);
	return 0;
}
		    
int main(int argc, char **argv) {
	struct tep_handle *tep;
	struct trace_seq seq;
	struct tep_event *sched_switch;
	int pid = getpid();
	char buf[24];
	char *filter = NULL;

	tep = tracefs_local_events(NULL);
	tep_set_long_size(tep, tep_get_header_page_size(tep));
	sched_switch = tep_find_event_by_name(tep, "sched", "sched_switch");
	sprintf(buf, "%d", pid);
	errno = 0;
	tracefs_filter_string_append(sched_switch, &filter, TRACEFS_FILTER_COMPARE,
				     "prev_pid", TRACEFS_COMPARE_EQ, buf);
	tracefs_filter_string_append(sched_switch, &filter, TRACEFS_FILTER_AND, NULL, 0, NULL);
	tracefs_filter_string_append(sched_switch, &filter, TRACEFS_FILTER_COMPARE,
				     "next_pid", TRACEFS_COMPARE_EQ, buf);
	if (errno) {
		printf("Failed filter\n");
		exit(-1);
	}
	tracefs_event_filter_apply(NULL, sched_switch, filter);
	free(filter);

	tracefs_trace_off(NULL);
	tracefs_event_disable(NULL, NULL, NULL);
	tracefs_instance_file_write(NULL, "trace", "");
	tracefs_event_enable(NULL, "sched", NULL);
	tracefs_trace_on(NULL);
	sleep(1);
	tracefs_trace_off(NULL);
	tracefs_load_cmdlines(NULL, tep);

	tracefs_event_filter_clear(NULL, sched_switch);

	trace_seq_init(&seq);
	tracefs_iterate_raw_events(tep, NULL, NULL, 0, callback, &seq);
	trace_seq_destroy(&seq);

	return 0;
}


