#define _GNU_SOURCE
#include <stdlib.h>
#include <unistd.h>
#include <tracefs.h>
#include <sched.h>
#include <errno.h>

static int callback(struct tep_event *event, struct tep_record *record,
		    int cpu, void *data)
{
	struct trace_seq *seq = data;

	trace_seq_reset(seq);
	tep_print_event(event->tep, seq, record, "%6.1000d", TEP_PRINT_TIME);
	trace_seq_printf(seq, " [%03d] ", cpu);
	tep_print_event(event->tep, seq, record, "%s-%d %s %s\n",
			TEP_PRINT_COMM, TEP_PRINT_PID,
			TEP_PRINT_NAME, TEP_PRINT_INFO);
	trace_seq_do_printf(seq);
	return 0;
}
		    
int main(int argc, char **argv) {
	struct tep_handle *tep;
	struct trace_seq seq;

	tep = tracefs_local_events(NULL);
	tep_set_long_size(tep, tep_get_header_page_size(tep));

	tracefs_function_filter(NULL, "*lock*", NULL, TRACEFS_FL_RESET | TRACEFS_FL_CONTINUE);
	tracefs_function_notrace(NULL, "*clock*", NULL, 0);
	tracefs_trace_off(NULL);
	tracefs_tracer_set(NULL, TRACEFS_TRACER_FUNCTION);
	tracefs_trace_on(NULL);
	sleep(1);
	tracefs_trace_off(NULL);
	tracefs_load_cmdlines(NULL, tep);

	trace_seq_init(&seq);
	tracefs_iterate_raw_events(tep, NULL, NULL, 0, callback, &seq);
	trace_seq_destroy(&seq);
	tracefs_tracer_clear(NULL);

	return 0;
}


